@@ -75,7 +75,8 @@ sub new {
 } #new
 
 sub new_with_same_permissions {
-    return new Yote::Obj();
+    my( $self, $args ) = @_;
+    return new Yote::Obj($args);
 } #new_with_same_permissions
 
 #
@@ -89,6 +90,14 @@ sub _init {}
 #
 sub _load {}
 
+#
+# May be called for the client to get lots of data at once
+# rather than asking piecemeal
+#
+sub precache {
+    my( $self, $data, $account ) = @_;
+} #precache
+
 # ------------------------------------------------------------------------------------------
 #      * UTILITY METHODS *
 # ------------------------------------------------------------------------------------------
@@ -201,7 +210,7 @@ sub _container_type {
 
 sub _count {
     my( $self, $args ) = @_;
-    if( ref( $args ) ) {
+    if( ref( $args ) ) { # TODO : standarize
 	return Yote::ObjProvider::count( $self->{DATA}{$args->{name}}, $args );
     }
     return Yote::ObjProvider::count( $self->{DATA}{$args} );
@@ -421,7 +430,7 @@ sub list_fetch {
 sub paginate {
     my( $self, $args, $account ) = @_;
     die "Access Error" unless $self->_check_access( $account, 0, $args->{ name } );
-    return Yote::ObjProvider::paginate( $self->{DATA}{ $args->{name} }, $args );
+    return $self->_paginate( $args );
 } #paginate
 
 sub remove_from {
