@@ -6,12 +6,12 @@ package Yote::RootObj;
 use strict;
 use warnings;
 
-use base 'Yote::Obj';
+use parent 'Yote::Obj';
 
 sub new_with_same_permissions {
-    my( $self, $dummy, $account ) = @_;
+    my( $self, $args, $account ) = @_;
     die "Permissions Error" unless $self->_check_access( $account, 1, '' );
-    return new Yote::RootObj();
+    return new Yote::RootObj( $args );
 } #new_with_same_permissions
 
 # only root may edit any fields. public fields are readable by all.
