@@ -428,7 +428,7 @@ sub __process_command {
 	# The token is stored with the IP address of the client and both
 	# are used to verify the token.
 	#
-	my $guest_token =  $root->check_guest_token( $command->{gt} );
+	my $guest_token =  $root->check_guest_token( $command->{e}{ REMOTE_ADDR }, $command->{gt} ) || $root->guest_token( $command->{e}{ REMOTE_ADDR } );
 	$command->{e}{GUEST_TOKEN} = $guest_token;
 
 	#
@@ -911,6 +911,7 @@ sub minify_dir {
     #
     my $minidir = "$source_root/_js";
     my $minifile = "$root/$minidir/mini.js";
+    my $debugfile = "$root/$minidir/maxi.js";
     
     if( ! -d "$root/$minidir" ) {
 	mkdir( "$root/$minidir" ); 
@@ -928,20 +929,21 @@ sub minify_dir {
 	}
     }
     my $minitime = -e $minifile ? stat($minifile)->mtime : 0;
-
     if( ! -f $minifile || $minitime < $latest_time ) {
-	my $buf = '';
+	my $mini_buf = '';
+	my $debug_buf = '';
 	# make sure base jquery comes first, followed by other jquery
 	# make sure that yote comes before yote.util
         for my $f (sort { ( $a =~ /jquery(-[0-9.]*)?(\.min)?\.js$/ || ($a =~ /jquery/ && $b !~ /jquery/ ) || $b =~ /yote.util/ ) ? -1 : 1
 		   } @js_files) {
 	    my $js = read_file( $f );
-	    $buf .= $f =~ /\.min\.js$/ ? $js : JavaScript::Minifier::minify(input => $js);
+	    $mini_buf .= $f =~ /\.min\.js$/ ? $js : JavaScript::Minifier::minify(input => $js);
+	    $debug_buf .= "$js\n";
 	}
-	open( my $OUT, '>', $minifile);
-	print $OUT $buf;
-	close( $OUT );
+	write_file( $minifile, $mini_buf );
+	write_file( $debugfile, $debug_buf );
     }
+
     return "$minidir/mini.js";
 } #minify_dir
 
